/*
 * Copyright 2020, TeamDev. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Redistribution and use in source and/or binary forms, with or without
 * modification, must retain the above copyright notice and the following
 * disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import 'package:firebase/firebase_io.dart' as fb;
import 'package:spine_client/firebase_client.dart';
import 'package:spine_client/src/url.dart';

/// A [FirebaseClient] based on the Firebase REST API.
///
/// This implementation does not have platform limitations.
///
/// See `WebFirebaseClient` for a web-specific implementation.
///
/// Note: the [childAdded], [childChanged] and [childRemoved] event streams are not supported by
/// this client implementation. It thus does not support Spine subscriptions. If you need the
/// subscriptions functionality, consider switching to the Firebase Admin SDK based client.
///
class RestClient implements FirebaseClient {

    static const _doesNotSupportSubscriptions =
        "REST Firebase client does not support the subscriptions.";

    final fb.FirebaseClient _client;
    final String _databaseUrl;

    /// Creates a new [RestClient] which connects to the database on the given [_databaseUrl]
    /// with the given REST API [_client].
    RestClient(this._client, this._databaseUrl);

    @override
    Stream<String> get(String path) async* {
        var root = await _client.get(Url.from(_databaseUrl, '${path}.json').stringUrl);
        if (root == null) {
            return;
        }
        for (var element in root.values) {
            yield element.toString();
        }
    }

    @override
    Stream<String> childAdded(String path) {
      throw UnimplementedError(_doesNotSupportSubscriptions);
    }

    @override
    Stream<String> childChanged(String path) {
        throw UnimplementedError(_doesNotSupportSubscriptions);
    }

    @override
    Stream<String> childRemoved(String path) {
        throw UnimplementedError(_doesNotSupportSubscriptions);
    }
}
